@@ -1,12 +1,10 @@
 from zeno.api import (
     DistillReturn,
-    InferenceReturn,
     MetricReturn,
     ModelReturn,
     ZenoOptions,
     ZenoParameters,
     distill,
-    inference,
     metric,
     model,
 )
@@ -14,14 +12,12 @@ from zeno.runner import get_server, zeno
 
 __all__ = [
     "DistillReturn",
-    "InferenceReturn",
     "MetricReturn",
     "ModelReturn",
     "ZenoOptions",
     "ZenoParameters",
     "get_server",
     "distill",
-    "inference",
     "metric",
     "model",
     "zeno",
