@@ -3,7 +3,6 @@
 import functools
 from typing import Any, Callable, Dict, List, Union
 
-from gradio.interface import IOComponent  # type: ignore
 from numpy.typing import NDArray
 from pandas import DataFrame, Series
 from pydantic import BaseModel
@@ -54,6 +53,7 @@ class ZenoParameters(BaseModel):
     cache_path: str = ""
     calculate_histogram_metrics = True
     editable: bool = True
+    multiprocessing: bool = True
     serve: bool = True
     samples: int = 30
     port: int = 8000
@@ -100,30 +100,9 @@ class MetricReturn(BaseModel):
 
     Args:
         metric (float): Average metric over subset of data
-        variance (float): Variance of metric over subset of data
     """
 
     metric: float
-    variance: Union[float, None] = None
-
-    class Config:
-        arbitrary_types_allowed = True
-
-
-class InferenceReturn(BaseModel):
-    """Return type for inference UI functions.
-    Take a look at the Gradio documentation for details.
-
-    Args:
-        input_components (List[Blocks]): List of input components.
-        output_components (Blocks): Output component.
-        input_columns (List[str]): List of input column names matching the
-        input components.
-    """
-
-    input_components: List[IOComponent]
-    output_component: IOComponent
-    input_columns: List[str]
 
     class Config:
         arbitrary_types_allowed = True
@@ -169,7 +148,7 @@ def metric(func: Callable[[DataFrame, ZenoOptions], MetricReturn]):
     Args:
         func (Callable[[DataFrame, ZenoOptions], MetricReturn]):
             A metric function that takes a DataFrame and ZenoOptions and
-            returns a MetricReturn with an average metric value and optional variance.
+            returns a MetricReturn with an average metric value and optional error rate.
     """
 
     @functools.wraps(func)
@@ -178,19 +157,3 @@ def metric(func: Callable[[DataFrame, ZenoOptions], MetricReturn]):
 
     _wrapper.metric_function = True  # type: ignore
     return _wrapper
-
-
-def inference(func: Callable[[ZenoOptions], InferenceReturn]):
-    """A decorator function for inference UI using Gradio.
-
-    Args:
-        func (Callable[[ZenoOptions], InferenceReturn]): Function that returns a set of
-        Gradio components and input columns to scaffold an inference UI.
-    """
-
-    @functools.wraps(func)
-    def _wrapper(*args, **kwargs):
-        return func(*args, **kwargs)
-
-    _wrapper.inference_function = True  # type: ignore
-    return _wrapper
