@@ -12,21 +12,21 @@ from pathlib import Path
 from typing import Callable, Dict, List, Optional, Union
 
 import pandas as pd
-from pandas import DataFrame  # type: ignore
-from pathos.multiprocessing import ProcessingPool as Pool  # type: ignore
+from pandas import DataFrame
+from pathos.multiprocessing import ProcessingPool as Pool
 
 from zeno.api import (
     DistillReturn,
-    InferenceReturn,
     MetricReturn,
     ModelReturn,
     ZenoOptions,
     ZenoParameters,
 )
 from zeno.classes.base import DataProcessingReturn, MetadataType, ZenoColumnType
-from zeno.classes.classes import MetricKey, TableRequest, ZenoColumn
+from zeno.classes.classes import MetricKey, PlotRequest, TableRequest, ZenoColumn
 from zeno.classes.report import Report
-from zeno.classes.slice import FilterIds, FilterPredicateGroup, Slice, SliceMetric
+from zeno.classes.slice import FilterIds, FilterPredicateGroup, GroupMetric, Slice
+from zeno.classes.tag import Tag, TagMetricKey
 from zeno.processing.data_processing import (
     postdistill_data,
     predistill_data,
@@ -34,7 +34,7 @@ from zeno.processing.data_processing import (
 )
 from zeno.processing.filtering import filter_table
 from zeno.util import (
-    getMetadataType,
+    get_metadata_type,
     load_series,
     read_functions,
     read_metadata,
@@ -55,10 +55,12 @@ class ZenoBackend(object):
         self.data_path = self.params.data_path
         self.label_path = self.params.label_path
         self.cache_path = self.params.cache_path
+        self.multiprocessing = self.params.multiprocessing
         self.editable = self.params.editable
         self.samples = self.params.samples
         self.view = self.params.view
         self.calculate_histogram_metrics = self.params.calculate_histogram_metrics
+        self.model_names = self.params.models
 
         self.df = read_metadata(self.metadata)
         self.tests = read_functions(self.functions)
@@ -82,9 +84,6 @@ class ZenoBackend(object):
         self.predict_function: Optional[
             Callable[[str], Callable[[DataFrame, ZenoOptions], ModelReturn]]
         ] = None
-        self.inference_function: Optional[
-            Callable[[ZenoOptions], InferenceReturn]
-        ] = None
         self.gradio_input_columns: List[str] = []
 
         self.status: str = "Initializing"
@@ -93,15 +92,16 @@ class ZenoBackend(object):
         self.slices: Dict[str, Slice] = read_pickle(
             "slices.pickle", self.cache_path, {}
         )
-
-        if self.params.models and os.path.isdir(self.params.models[0]):
-            self.model_paths = [
-                os.path.join(self.params.models[0], m)
-                for m in os.listdir(self.params.models[0])
-            ]
-        else:
-            self.model_paths = self.params.models  # type: ignore
-        self.model_names = [os.path.basename(p).split(".")[0] for p in self.model_paths]
+        self.tags: Dict[str, Tag] = read_pickle("tags.pickle", self.cache_path, {})
+        if "All Instances" not in self.slices:
+            orig_slices = self.slices
+            all_instance = Slice(
+                slice_name="All Instances",
+                folder="",
+                filter_predicates=FilterPredicateGroup(predicates=[], join=""),
+            )
+            self.slices = {"All Instances": all_instance}
+            self.slices.update(orig_slices)
 
         self.__setup_dataframe(
             self.params.id_column, self.params.data_column, self.params.label_column
@@ -122,11 +122,19 @@ class ZenoBackend(object):
 
     def __setup_dataframe(self, id_column: str, data_column: str, label_column: str):
         if data_column != "":
-            self.data_column = ZenoColumn(
-                column_type=ZenoColumnType.METADATA,
-                metadata_type=getMetadataType(self.df[data_column]),
-                name=data_column,
-            )
+            if data_column != id_column:
+                self.data_column = ZenoColumn(
+                    column_type=ZenoColumnType.METADATA,
+                    metadata_type=get_metadata_type(self.df[data_column]),
+                    name=data_column,
+                )
+            else:  # make sure id and data column are different
+                self.df["data"] = self.df[data_column]
+                self.data_column = ZenoColumn(
+                    column_type=ZenoColumnType.METADATA,
+                    metadata_type=get_metadata_type(self.df["data"]),
+                    name="data",
+                )
         else:
             self.data_column = ZenoColumn(
                 column_type=ZenoColumnType.METADATA,
@@ -137,7 +145,7 @@ class ZenoBackend(object):
         if label_column != "":
             self.label_column = ZenoColumn(
                 column_type=ZenoColumnType.METADATA,
-                metadata_type=getMetadataType(self.df[label_column]),
+                metadata_type=get_metadata_type(self.df[label_column]),
                 name=label_column,
             )
         else:
@@ -155,7 +163,7 @@ class ZenoBackend(object):
             )
             self.df[str(self.id_column)].astype(str)
         else:
-            self.df.reset_index(inplace=True)
+            self.df = self.df.reset_index()
             self.id_column = ZenoColumn(
                 column_type=ZenoColumnType.METADATA,
                 metadata_type=MetadataType.OTHER,
@@ -165,13 +173,13 @@ class ZenoBackend(object):
         self.columns: List[ZenoColumn] = []
         self.complete_columns: List[ZenoColumn] = []
 
-        self.df.set_index(str(self.id_column), inplace=True, drop=False)
+        self.df = self.df.set_index(str(self.id_column), drop=False)
         # Set index name to None to prevent name overlaps w/ columns.
         self.df.index.name = None
         for metadata_col in self.df.columns:
             col = ZenoColumn(
                 column_type=ZenoColumnType.METADATA,
-                metadata_type=getMetadataType(self.df[metadata_col]),
+                metadata_type=get_metadata_type(self.df[metadata_col]),
                 name=str(metadata_col),
             )
             self.columns.append(col)
@@ -193,12 +201,6 @@ class ZenoBackend(object):
                     self.predistill_functions[test_fn.__name__] = test_fn
             if hasattr(test_fn, "metric_function"):
                 self.metric_functions[test_fn.__name__] = test_fn
-            if hasattr(test_fn, "inference_function"):
-                if self.inference_function is None:
-                    self.inference_function = test_fn  # type: ignore
-                else:
-                    print("ERROR: Multiple model functions found, can only have one")
-                    sys.exit(1)
 
     def start_processing(self):
         """Parse testing files, distill, and run inference."""
@@ -222,7 +224,9 @@ class ZenoBackend(object):
                     )
                 )
 
-        self.__thread = threading.Thread(target=asyncio.run, args=(self.__process(),))
+        self.__thread = threading.Thread(
+            target=asyncio.run, args=(self.__process(),), daemon=True
+        )
         self.__thread.start()
 
     async def __process(self):
@@ -254,7 +258,7 @@ class ZenoBackend(object):
                 c_hash = str(out.column)
                 self.df.loc[:, c_hash] = out.output
                 self.df[c_hash] = self.df[c_hash].convert_dtypes()
-                out.column.metadata_type = getMetadataType(self.df[c_hash])
+                out.column.metadata_type = get_metadata_type(self.df[c_hash])
                 self.complete_columns.append(out.column)
 
     def __predistill(self) -> None:
@@ -270,27 +274,46 @@ class ZenoBackend(object):
             load_series(self.df, predistill_column, save_path)
 
             predistill_hash = str(predistill_column)
-            if self.df[predistill_hash].isnull().any():
+            if self.df[predistill_hash].isna().any():
                 predistill_to_run.append(predistill_column)
             else:
                 self.df[predistill_hash] = self.df[predistill_hash].convert_dtypes()
-                predistill_column.metadata_type = getMetadataType(
+                predistill_column.metadata_type = get_metadata_type(
                     self.df[predistill_hash]
                 )
                 self.complete_columns.append(predistill_column)
 
         if len(predistill_to_run) > 0:
-            with Pool() as pool:
-                predistill_outputs = pool.map(
-                    predistill_data,
-                    [self.predistill_functions[col.name] for col in predistill_to_run],
-                    [col for col in predistill_to_run],
-                    [self.zeno_options] * len(predistill_to_run),
-                    [self.cache_path] * len(predistill_to_run),
-                    [self.df] * len(predistill_to_run),
-                    [self.batch_size] * len(predistill_to_run),
-                    range(len(predistill_to_run)),
-                )
+            if self.multiprocessing:
+                with Pool() as pool:
+                    predistill_outputs = pool.map(
+                        predistill_data,
+                        [
+                            self.predistill_functions[col.name]
+                            for col in predistill_to_run
+                        ],
+                        [col for col in predistill_to_run],
+                        [self.zeno_options] * len(predistill_to_run),
+                        [self.cache_path] * len(predistill_to_run),
+                        [self.df] * len(predistill_to_run),
+                        [self.batch_size] * len(predistill_to_run),
+                        range(len(predistill_to_run)),
+                    )
+                    self.__set_data_processing_returns(predistill_outputs)
+            else:
+                predistill_outputs = []
+                for i, predistill in enumerate(predistill_to_run):
+                    predistill_outputs.append(
+                        predistill_data(
+                            self.predistill_functions[predistill.name],
+                            predistill,
+                            self.zeno_options,
+                            self.cache_path,
+                            self.df,
+                            self.batch_size,
+                            i,
+                        )
+                    )
                 self.__set_data_processing_returns(predistill_outputs)
 
     def __inference(self):
@@ -298,15 +321,12 @@ class ZenoBackend(object):
 
         # Check if we need to run inference since Pool is expensive
         models_to_run = []
-        for model_path in self.model_paths:
-            model_name = os.path.basename(model_path).split(".")[0]
+        for model_name in self.model_names:
             model_column = ZenoColumn(
-                column_type=ZenoColumnType.OUTPUT,
-                name=model_name,
+                column_type=ZenoColumnType.OUTPUT, name="output", model=model_name
             )
             embedding_column = ZenoColumn(
-                column_type=ZenoColumnType.EMBEDDING,
-                name=model_name,
+                column_type=ZenoColumnType.EMBEDDING, name="embedding", model=model_name
             )
             model_hash = str(model_column)
             embedding_hash = str(embedding_column)
@@ -317,14 +337,11 @@ class ZenoBackend(object):
             load_series(self.df, model_column, model_save_path)
             load_series(self.df, embedding_column, embedding_save_path)
 
-            if (
-                self.df[model_hash].isnull().any()
-                or self.df[embedding_hash].isnull().any()
-            ):
-                models_to_run.append(model_path)
+            if self.df[model_hash].isna().any():
+                models_to_run.append(model_name)
             else:
                 self.df[model_hash] = self.df[model_hash].convert_dtypes()
-                model_column.metadata_type = getMetadataType(self.df[model_hash])
+                model_column.metadata_type = get_metadata_type(self.df[model_hash])
                 self.complete_columns.append(model_column)
 
                 # Check if there were saved postdistill columns:
@@ -342,22 +359,37 @@ class ZenoBackend(object):
                     series = pd.read_pickle(f)
                     self.df.loc[:, str(col)] = series
                     self.df[str(col)] = self.df[str(col)].convert_dtypes()
-                    col.metadata_type = getMetadataType(self.df[str(col)])
+                    col.metadata_type = get_metadata_type(self.df[str(col)])
                     self.complete_columns.append(col)
 
-        if len(models_to_run) > 0:
-            with Pool() as pool:
-                inference_outputs = pool.map(
-                    run_inference,
-                    [self.predict_function] * len(models_to_run),
-                    [self.zeno_options] * len(models_to_run),
-                    [m for m in models_to_run],
-                    [self.cache_path] * len(models_to_run),
-                    [self.df] * len(models_to_run),
-                    [self.batch_size] * len(models_to_run),
-                    range(len(models_to_run)),
-                )
-                self.__set_data_processing_returns(inference_outputs)
+        if len(models_to_run) > 0 and self.predict_function is not None:
+            if self.multiprocessing:
+                with Pool() as pool:
+                    inference_outputs = pool.map(
+                        run_inference,
+                        [self.predict_function] * len(models_to_run),
+                        [self.zeno_options] * len(models_to_run),
+                        [m for m in models_to_run],
+                        [self.cache_path] * len(models_to_run),
+                        [self.df] * len(models_to_run),
+                        [self.batch_size] * len(models_to_run),
+                        range(len(models_to_run)),
+                    )
+            else:
+                inference_outputs = []
+                for i, model_name in enumerate(models_to_run):
+                    inference_outputs.append(
+                        run_inference(
+                            self.predict_function,
+                            self.zeno_options,
+                            model_name,
+                            self.cache_path,
+                            self.df,
+                            self.batch_size,
+                            i,
+                        )
+                    )
+            self.__set_data_processing_returns(inference_outputs)
 
     def __postdistill(self) -> None:
         """Run distill functions dependent on model outputs."""
@@ -382,53 +414,108 @@ class ZenoBackend(object):
 
             load_series(self.df, col_name, save_path)
 
-            if self.df[col_hash].isnull().any():
+            if self.df[col_hash].isna().any():
                 postdistill_to_run.append(col_name)
             else:
                 self.df[col_hash] = self.df[col_hash].convert_dtypes()
-                col_name.metadata_type = getMetadataType(self.df[col_hash])
+                col_name.metadata_type = get_metadata_type(self.df[col_hash])
                 self.complete_columns.append(col_name)
 
         if len(postdistill_to_run) > 0:
-            with Pool() as pool:
-                post_outputs = pool.map(
-                    postdistill_data,
-                    [self.postdistill_functions[e.name] for e in postdistill_to_run],
-                    [e.model for e in postdistill_to_run],
-                    [self.zeno_options] * len(postdistill_to_run),
-                    [self.cache_path] * len(postdistill_to_run),
-                    [self.df] * len(postdistill_to_run),
-                    [self.batch_size] * len(postdistill_to_run),
-                    range(len(postdistill_to_run)),
-                )
+            if self.multiprocessing:
+                with Pool() as pool:
+                    post_outputs = pool.map(
+                        postdistill_data,
+                        [
+                            self.postdistill_functions[e.name]
+                            for e in postdistill_to_run
+                        ],
+                        [e.model for e in postdistill_to_run],
+                        [self.zeno_options] * len(postdistill_to_run),
+                        [self.cache_path] * len(postdistill_to_run),
+                        [self.df] * len(postdistill_to_run),
+                        [self.batch_size] * len(postdistill_to_run),
+                        range(len(postdistill_to_run)),
+                    )
+            else:
+                post_outputs = []
+                for i, postdistill in enumerate(postdistill_to_run):
+                    post_outputs.append(
+                        postdistill_data(
+                            self.postdistill_functions[postdistill.name],
+                            postdistill.model if postdistill.model else "",
+                            self.zeno_options,
+                            self.cache_path,
+                            self.df,
+                            self.batch_size,
+                            i,
+                        )
+                    )
             self.__set_data_processing_returns(post_outputs)
 
     def get_metrics_for_slices(
         self,
         requests: List[MetricKey],
         filter_ids: Optional[FilterIds] = None,
-    ) -> List[SliceMetric]:
+    ) -> List[GroupMetric]:
         """Calculate result for each requested combination."""
 
-        return_metrics: List[SliceMetric] = []
+        return_metrics: List[GroupMetric] = []
         for metric_key in requests:
-
             # If we refresh, might not have columns for a slice.
             try:
                 filt_df = filter_table(
                     self.df, metric_key.sli.filter_predicates, filter_ids
                 )
             except pd.errors.UndefinedVariableError:
-                return_metrics.append(SliceMetric(metric=None, size=0))
+                return_metrics.append(GroupMetric(metric=None, size=0))
                 continue
 
             if metric_key.metric == "" or self.label_column.name == "":
-                return_metrics.append(SliceMetric(metric=None, size=filt_df.shape[0]))
+                return_metrics.append(GroupMetric(metric=None, size=filt_df.shape[0]))
             else:
                 metric = self.calculate_metric(
                     filt_df, metric_key.model, metric_key.metric
                 )
-                return_metrics.append(SliceMetric(metric=metric, size=filt_df.shape[0]))
+                return_metrics.append(GroupMetric(metric=metric, size=filt_df.shape[0]))
+        return return_metrics
+
+    def get_metrics_for_slices_and_tags(
+        self,
+        requests: List[MetricKey],
+        tag_ids: Optional[FilterIds] = None,
+        filter_ids: Optional[FilterIds] = None,
+        tag_list: Optional[List[str]] = None,
+    ) -> List[GroupMetric]:
+        """Calculate result for each requested combination."""
+        return_metrics: List[GroupMetric] = []
+        for metric_key in requests:
+            filt_df = filter_table(
+                self.df, metric_key.sli.filter_predicates, tag_ids, filter_ids, tag_list
+            )
+            if metric_key.metric == "" or self.label_column.name == "":
+                return_metrics.append(GroupMetric(metric=None, size=filt_df.shape[0]))
+            else:
+                metric = self.calculate_metric(
+                    filt_df, metric_key.model, metric_key.metric
+                )
+                return_metrics.append(GroupMetric(metric=metric, size=filt_df.shape[0]))
+        return return_metrics
+
+    def get_metrics_for_tags(self, requests: List[TagMetricKey]) -> List[GroupMetric]:
+        return_metrics: List[GroupMetric] = []
+        for tag_metric_key in requests:
+            filt_df = filter_table(self.df, None, tag_metric_key.tag.selection_ids)
+            if tag_metric_key.metric == "" or self.label_column.name == "":
+                return_metrics.append(GroupMetric(metric=None, size=filt_df.shape[0]))
+            else:
+                # if the tag is empty
+                if len(tag_metric_key.tag.selection_ids.ids) == 0:
+                    filt_df = self.df.iloc[0:0]
+                metric = self.calculate_metric(
+                    filt_df, tag_metric_key.model, tag_metric_key.metric
+                )
+                return_metrics.append(GroupMetric(metric=metric, size=filt_df.shape[0]))
         return return_metrics
 
     def calculate_metric(
@@ -438,10 +525,8 @@ class ZenoBackend(object):
             return None
 
         if model is not None:
-
             output_col = ZenoColumn(
-                column_type=ZenoColumnType.OUTPUT,
-                name=model,
+                column_type=ZenoColumnType.OUTPUT, name="output", model=model
             )
             output_hash = str(output_col)
 
@@ -495,6 +580,20 @@ class ZenoBackend(object):
         with open(os.path.join(self.cache_path, "folders.pickle"), "wb") as f:
             pickle.dump(self.folders, f)
 
+    def create_new_tag(self, req: Tag):
+        if not self.editable:
+            return
+        self.tags[req.tag_name] = req
+        with open(os.path.join(self.cache_path, "tags.pickle"), "wb") as f:
+            pickle.dump(self.tags, f)
+
+    def delete_tag(self, tag_name: str):
+        if not self.editable:
+            return
+        del self.tags[tag_name]
+        with open(os.path.join(self.cache_path, "tags.pickle"), "wb") as f:
+            pickle.dump(self.tags, f)
+
     def set_reports(self, reports: List[Report]):
         if not self.editable:
             return
@@ -516,50 +615,45 @@ class ZenoBackend(object):
         with open(os.path.join(self.cache_path, "slices.pickle"), "wb") as f:
             pickle.dump(self.slices, f)
 
-    def get_filtered_ids(self, req: FilterPredicateGroup):
-        return filter_table(self.df, req)[str(self.id_column)].to_json(orient="records")
+    def get_filtered_ids(self, req: PlotRequest):
+        return filter_table(self.df, req.filter_predicates, req.tag_ids)[
+            str(self.id_column)
+        ].to_json(orient="records")
 
     def get_filtered_table(self, req: TableRequest):
         """Return filtered table from list of filter predicates."""
-        filt_df = filter_table(self.df, req.filter_predicates, req.filter_ids)
+        filt_df = filter_table(
+            self.df, req.filter_predicates, req.filter_ids, req.tag_ids, req.tag_list
+        )
+        req_columns = [str(col) for col in req.columns]
+        if req.diff_columns:
+            filt_df = self.generate_diff_cols(filt_df, req.diff_columns)
+            req_columns.append("diff")
         if req.sort[0]:
             filt_df = filt_df.sort_values(str(req.sort[0]), ascending=req.sort[1])
         filt_df = filt_df.iloc[req.slice_range[0] : req.slice_range[1]].copy()
-
         if self.data_prefix != "":
             # Add data prefix to data column depending on type of data_path.
             filt_df.loc[:, str(self.data_column)] = (
                 self.data_prefix + filt_df[str(self.data_column)]
             )
+        return filt_df.loc[:, req_columns].to_json(orient="records")
 
-        return filt_df[[str(col) for col in req.columns]].to_json(orient="records")
+    def generate_diff_cols(self, df: DataFrame, diff_cols: List[ZenoColumn]):
+        col_1, col_2 = diff_cols[0], diff_cols[1]
+        if (
+            col_1.column_type != col_2.column_type
+            or col_1.metadata_type != col_2.metadata_type
+        ):
+            print("error: different column types!")
+            return df
 
-    def single_inference(self, *args):
-        """Run inference from Gradio inputs.
-        The first input to args is the model, while the rest are dynamic inputs
-        corresponding to the columns defined in self.gradio_input_columns.
+        # various metadata type difference
+        if col_1.metadata_type == MetadataType.CONTINUOUS:
+            df.loc[:, "diff"] = abs(
+                df[str(col_1)].astype(float) - df[str(col_2)].astype(float)
+            )
+        else:
+            df.loc[:, "diff"] = df[str(col_1)] != df[str(col_2)]
 
-        Returns:
-            any: output of the selected model.
-        """
-        if not self.predict_function:
-            return
-
-        # Get prediction function for selected model.
-        model_fn = self.predict_function(args[0])
-
-        # Get the columns that correspond to the inputs.
-        metadata_cols = [
-            c for c in self.columns if c.column_type == ZenoColumnType.METADATA
-        ]
-        metadata_cols_names = [c.name for c in metadata_cols]
-        cols = []
-        for c in self.gradio_input_columns:
-            if c in metadata_cols_names:
-                cols.append(str(metadata_cols[metadata_cols_names.index(c)]))
-
-        temp_df = pd.DataFrame(columns=cols)
-        temp_df.loc[0] = args[1:]  # type: ignore
-        out = model_fn(temp_df, self.zeno_options)
-
-        return out.model_output
+        return df
