@@ -5,7 +5,6 @@ from typing import List, Optional, Tuple, Union
 from pydantic import BaseModel
 
 from zeno.classes.base import CamelModel, ZenoColumn
-
 from zeno.classes.slice import FilterIds, FilterPredicateGroup, Slice
 
 
@@ -16,8 +15,7 @@ class ZenoSettings(CamelModel):
     data_column: ZenoColumn
     samples: int
     calculate_histogram_metrics: bool
-    inference_view: bool
-    totalSize: int
+    total_size: int
 
 
 class ZenoVariables(CamelModel):
@@ -40,15 +38,25 @@ class MetricKey(CamelModel):
 
 class MetricRequest(CamelModel):
     metric_keys: List[MetricKey]
+    tag_ids: Optional[FilterIds] = None
     filter_ids: Optional[FilterIds] = None
+    tag_list: Optional[List[str]] = None
 
 
 class TableRequest(CamelModel):
     columns: List[ZenoColumn]
+    diff_columns: List[ZenoColumn]
     slice_range: List[int]
     filter_predicates: FilterPredicateGroup
-    sort: Tuple[Union[ZenoColumn, None], bool]
+    sort: Tuple[Union[ZenoColumn, str, None], bool]
+    tag_ids: FilterIds
     filter_ids: Optional[FilterIds] = None
+    tag_list: List[str]
+
+
+class PlotRequest(CamelModel):
+    filter_predicates: FilterPredicateGroup
+    tag_ids: FilterIds
 
 
 class EmbedProject2DRequest(CamelModel):
