@@ -19,12 +19,14 @@ class HistogramRequest(CamelModel):
     filter_predicates: Union[None, FilterPredicateGroup] = None
     model: Union[None, str] = None
     metric: Union[None, str] = None
+    tag_ids: Optional[FilterIds] = None
     filter_ids: Optional[FilterIds] = None
+    tag_list: Optional[List[str]] = None
 
 
 class StringFilterRequest(CamelModel):
     column: ZenoColumn
     filter_string: str
-    selection_type: str
+    is_regex: bool
     case_match: bool
     whole_word_match: bool
