@@ -1,5 +1,5 @@
 """Functions for parsing filter predicates and filtering dataframes"""
-from typing import Optional
+from typing import List, Optional
 
 import pandas as pd
 from pandas import DataFrame
@@ -27,21 +27,24 @@ def get_filter_string(filter: FilterPredicateGroup) -> str:
                 filt = filt + get_filter_string(f)
                 filt = filt + ")"
         elif isinstance(f, FilterPredicate):
-            if f.operation == "match":
-                filt = (
-                    filt
-                    + "{} (`{}`.astype('string').str.contains('{}', na=False,".format(
-                        f.join, f.column, f.value
-                    )
-                    + "regex=False, case=False))"
-                )
-            elif f.operation == "match (regex)":
-                filt = (
-                    filt
-                    + "{} (`{}`.astype('string').str.contains('{}', na=False))".format(
-                        f.join, f.column, f.value
-                    )
-                )
+            if "match" in f.operation:
+                is_regex = "re" in f.operation
+                is_case = "ca" in f.operation
+                is_whole = "w" in f.operation
+
+                if is_whole:
+                    f.value = f"\\b{f.value}\\b" if is_regex else f'"{f.value}"'
+
+                filt_string = f"{f.join} (`{f.column}`.str.contains(\
+                    r'{f.value}', na=False, regex={is_regex}, case={is_case}))"
+
+                if (not is_regex) and is_whole:
+                    filt_string = f"{f.join} (`{f.column}`=={f.value})"
+
+                try:
+                    filt += filt_string
+                except Exception as e:
+                    print("Invalid Regex Error: ", e)
             else:
                 try:
                     val = str(float(f.value))
@@ -60,20 +63,34 @@ def get_filter_string(filter: FilterPredicateGroup) -> str:
 
 
 def filter_table(
-    df,
-    filter_predicates: FilterPredicateGroup,
-    filter_ids: Optional[FilterIds] = None,
+    main_df,
+    filter_predicates: Optional[FilterPredicateGroup] = None,
+    list_ids_first: Optional[FilterIds] = None,
+    list_ids_second: Optional[FilterIds] = None,
+    tag_list: Optional[List[str]] = None,
 ) -> pd.DataFrame:
+    all_indicies = []
+    if list_ids_first is not None and len(list_ids_first.ids) > 0:
+        all_indicies += list_ids_first.ids
+    if list_ids_second is not None and len(list_ids_second.ids) > 0:
+        all_indicies += list_ids_second.ids
     # if we have ids, filter them out now!
-    if filter_ids is not None and len(filter_ids.ids) > 0:
+    if len(all_indicies) > 0:
+        # make sure the ids we are querying exist
+        existing_ids = main_df.index.intersection(all_indicies)
         # this is fast because the index is set to ids
-        df = df.loc[filter_ids.ids]
+        main_df = main_df.loc[existing_ids]
 
-    final_filter = get_filter_string(filter_predicates)
-    if len(final_filter) > 0:
-        return df.query(final_filter, engine="python")
-    else:
-        return df
+    # empty selected tags so always return empty table
+    if len(all_indicies) == 0 and tag_list is not None and len(tag_list) > 0:
+        return main_df.iloc[0:0]
+
+    if filter_predicates is not None:
+        final_filter = get_filter_string(filter_predicates)
+        if len(final_filter) > 0:
+            return main_df.query(final_filter, engine="python")
+
+    return main_df
 
 
 def filter_table_single(df: DataFrame, col: ZenoColumn, bucket: HistogramBucket):
