@@ -38,7 +38,7 @@ def histogram_buckets(
                 ret_hist.append(HistogramBucket(bucket=k))
             res.append(ret_hist)
         elif col.metadata_type == MetadataType.CONTINUOUS:
-            ret_hist: List[HistogramBucket] = []  # type: ignore
+            ret_hist: List[HistogramBucket] = []
             df_col = df_col.fillna(0)
             bins = np.histogram_bin_edges(df_col, bins=num_bins)
             for i in range(len(bins) - 1):
@@ -66,7 +66,9 @@ def histogram_buckets(
 def histogram_counts(df: pd.DataFrame, req: HistogramRequest) -> List[List[int]]:
     """Calculate count for each bucket in each column histogram."""
     if req.filter_predicates is not None:
-        filt_df = filter_table(df, req.filter_predicates, req.filter_ids)
+        filt_df = filter_table(
+            df, req.filter_predicates, req.tag_ids, req.filter_ids, req.tag_list
+        )
     else:
         filt_df = df
 
@@ -123,7 +125,9 @@ def histogram_metrics(
         return []
 
     if req.filter_predicates is not None:
-        filt_df = filter_table(df, req.filter_predicates, req.filter_ids)
+        filt_df = filter_table(
+            df, req.filter_predicates, req.tag_ids, req.filter_ids, req.tag_list
+        )
     else:
         filt_df = df
 
@@ -145,14 +149,14 @@ def histogram_metrics(
 def filter_by_string(df: pd.DataFrame, req: StringFilterRequest) -> List[str]:
     """Filter the table based on a string filter request."""
     short_ret: List[str] = []
-    type = req.selection_type
+    regex = req.is_regex
     keyword = req.filter_string
     col_type = req.column
     case_match = req.case_match
     whole_word_match = req.whole_word_match
 
     # string search
-    if type == "string":
+    if not regex:
         col = df[str(col_type)].dropna().astype(str)
 
         if not case_match:
@@ -179,16 +183,16 @@ def filter_by_string(df: pd.DataFrame, req: StringFilterRequest) -> List[str]:
         flag = 0 if case_match else re.IGNORECASE
         keyword = f"\\b{keyword}\\b" if whole_word_match else keyword
         try:
-            query_string = f"{col_type}.str.contains(r'{keyword}', flags=@flag)"
+            query_string = f"`{col_type}`.str.contains(r'{keyword}', flags=@flag)"
             ret = df.query(query_string)[str(col_type)].head().tolist()
         except Exception as e:
             print("Invalid Regex Error: ", e)
             return short_ret
 
         for r in ret:
-            idx = re.search(keyword, r, flags=flag)  # type: ignore
+            idx = re.search(keyword, r, flags=flag)
             if idx is not None:
-                idx = idx.start()  # type: ignore
+                idx = idx.start()
                 loc_str = r[0 if idx < 20 else idx - 20 : idx + 20]
                 if len(r) > 40 + len(keyword):
                     if idx - 20 > 0:
